SystemOrganization addCategory: #'Athens-Morphic'!

!EllipseMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 7/1/2014 22:39'!
asAthensShapeOn: anAthensCanvas
	| radius width |
	self bounds area > 0
		ifFalse: [ ^ self ].
	width := self bounds width.
	radius := width / 2.
	^ anAthensCanvas
		createPath: [ :builder | 
			builder
				absolute;
				moveTo: width @ radius;
				cwArcTo: 0 @ radius angle: Float pi;
				cwArcTo: width @ radius angle: Float pi ]! !

!EllipseMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 7/1/2014 22:39'!
drawOnAthensCanvas: anAthensCanvas
	| border ellipse |
	self bounds area > 0
		ifFalse: [ ^ self ].
	border := self borderStyle.
	ellipse := self asAthensShapeOn: anAthensCanvas.
	anAthensCanvas setPaint: self fillStyle.
	anAthensCanvas setShape: ellipse.
	anAthensCanvas pathTransform
		restoreAfter: [ 
			| t |
			t := anAthensCanvas pathTransform.
			t translateBy: self bounds origin.
			t scaleBy: 1 @ (self bounds height / self bounds width).
			anAthensCanvas setShape: ellipse.
			anAthensCanvas draw.
			(anAthensCanvas setStrokePaint: border color) width: border width.
			anAthensCanvas draw ]! !

!PasteUpMorph methodsFor: '*Athens-Morphic' stamp: 'IgorStasenko 4/21/2011 14:33'!
drawOnAthensCanvas: aCanvas 
	"Draw in order:
	- background color
	- background sketch, if any
	- Update and draw the turtleTrails form. See the comment in updateTrailsForm.

	Later (in drawSubmorphsOn:) I will skip drawing the background sketch."

	"draw background fill"
	super drawOnAthensCanvas: aCanvas.

	backgroundMorph ifNotNil: [
		aCanvas clipBy: self clippingBounds
			during: [ aCanvas fullDrawMorph: backgroundMorph ]
	]				
! !

!StringMorph methodsFor: '*Athens-Morphic' stamp: 'IgorStasenko 10/3/2012 17:27'!
drawOnAthensCanvas: canvas
	| pt bnd gap |
"	bnd := self bounds.
	gap := self layoutInset.
"	bnd := self bounds topLeft + self layoutInset.
	 
	font := self fontToUse.
	
	canvas pathTransform restoreAfter: [
		canvas pathTransform translateBy: bnd.
		canvas pathTransform translateX: 0 Y: font getPreciseAscent.
		
		canvas 
			setPaint: color;
			setFont: self fontToUse;
			drawString: contents.
	]! !

!EmbeddedMenuMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/11/2015 13:51'!
drawOnAthensCanvas: anAthensCanvas
	"Draw the receiver on the canvas."

	self perform: #drawOnAthensCanvas: withArguments: {anAthensCanvas} inSuperclass: Morph.
	self hasKeyboardFocus ifTrue: [self drawKeyboardFocusOnAthensCanvas: anAthensCanvas]! !

!LabelMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/23/2015 13:48'!
drawOnAthensCanvas: aCanvas
	"Draw based on enablement."

	| pc |
	pc := self owner
		ifNil: [ self paneColor ]
		ifNotNil: [ 
			self owner color isTransparent
				ifTrue: [ self owner paneColor ]
				ifFalse: [ self owner color ] ].
	aCanvas
		clipBy: self bounds
		during: [ 
			aCanvas pathTransform
				restoreAfter: [ 
					aCanvas pathTransform translateX: self left Y: self top + self fontToUse ascent.
					aCanvas
						setFont: self fontToUse;
						setPaint:
								(self enabled
										ifTrue: [ self color ]
										ifFalse: [ pc muchDarker ]);
						drawString: self contents ] ]! !

!MulticolumnLazyListMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/17/2015 16:11'!
athensDisplay: items atRow: row on: anAthensCanvas
	| drawBounds backgroundColor |
	backgroundColor := self backgroundColorForRow: row.
	drawBounds := self drawBoundsForRow: row.
	drawBounds := drawBounds
		intersect: self bounds
		ifNone: [ 
			"oh well"
			^ self ].
	items
		with: (1 to: items size)
		do: [ :item :index | 
			"move the bounds to the right at each step"
			index > 1
				ifTrue: [ drawBounds := drawBounds left: drawBounds left + listSource gapSize + (columnWidths at: index - 1) ].
			anAthensCanvas
				morphicDrawString: item asString
				at: drawBounds topLeft
				font: self font
				color: color]! !

!MulticolumnLazyListMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/17/2015 16:13'!
drawOnAthensCanvas: anAthensCanvas
	self getListSize = 0 ifTrue:[ ^self ].

	self setColumnWidthsFor: anAthensCanvas.
	self adjustWidth.
	
	super drawOnAthensCanvas: anAthensCanvas! !

!TabBar methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/7/2015 22:18'!
drawLinesOnAthensCanvas: anAthensCanvas
	| path |
	path := anAthensCanvas
		createPath: [ :builder | 
			builder moveTo: self bottomLeft + (0 @ -1).
			builder lineTo: self bottomRight + (0 @ -1) ].
	(anAthensCanvas setStrokePaint: self borderColor) width: 5.
	anAthensCanvas drawShape: path! !

!TabBar methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/7/2015 22:18'!
drawLinesWithSelection: anAthensCanvas
	| path |
	path := anAthensCanvas
		createPath: [ :builder | 
			builder absolute.
			builder moveTo: self bottomLeft + (0 @ -1).
			builder lineTo: self selectedTab bottomLeft.
			builder moveTo:  self selectedTab  bottomRight.
			builder lineTo: self bottomRight + (0 @ -1)].
	(anAthensCanvas setStrokePaint: self borderColor) width: 1.
	anAthensCanvas drawShape: path.

	path := anAthensCanvas
		createPath: [ :builder | 
			builder absolute.			
			builder moveTo: self selectedTab bottomLeft.
			builder lineTo: self selectedTab  bottomRight].
	(anAthensCanvas setStrokePaint: self selectedColor) width:1.
	anAthensCanvas drawShape: path.
	! !

!TabBar methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/7/2015 22:06'!
drawSubmorphsOnAthensCanvas: anAthensCanvas
	super drawSubmorphsOnAthensCanvas: anAthensCanvas.
	self selectedTab
		ifNil: [ self drawLinesOnAthensCanvas: anAthensCanvas ]
		ifNotNil: [ self drawLinesWithSelection: anAthensCanvas ]! !

!CheckboxButtonMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/15/2015 12:17'!
drawOnAthensCanvas: anAthensCanvas
	"Draw the image for the current state."

	| img |
	anAthensCanvas setPaint: self fillStyle.
	anAthensCanvas drawShape: self bounds.
	anAthensCanvas setPaint: self borderStyle.
	anAthensCanvas drawShape: self bounds.
	img := self imageToUse.
	img
		ifNotNil: [ 
			anAthensCanvas setPaint: img.
			anAthensCanvas drawShape: (self innerBounds center - (img extent // 2) extent: img extent) ].
	((self state == #pressed or: [ self state == #repressed ]) and: [ image isNil ])
		ifTrue: [ 
			anAthensCanvas setPaint: (self paneColor alpha: 0.3).
			anAthensCanvas drawShape: self innerBounds ].
	(self enabled not and: [ self theme fadeCheckboxWhenDisabled ])
		ifTrue: [ 
			anAthensCanvas setPaint: (self paneColor alpha: 0.4).
			anAthensCanvas drawShape: self innerBounds ]! !

!AthensSurface methodsFor: '*Athens-Morphic' stamp: 'IgorStasenko 12/21/2013 02:16'!
displayOnMorphicCanvas: canvas at: aPoint
	"display receiver on morphic canvas"
	self subclassResponsibility! !

!PluggableSliderMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/6/2015 18:08'!
drawOnAthensCanvas: aCanvas 

	super drawOnAthensCanvas: aCanvas.
	aCanvas 
		morphicDrawString: self label
		in: self labelBounds
		font: self font
		color: self fontColor.
		
	
! !

Morph subclass: #AthensSceneView
	instanceVariableNames: 'surface scene session transform state keepRefreshing'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Morphic'!

!AthensSceneView commentStamp: '<historical>' prior: 0!
i am simple morph for representing some scene.

The scene is any object which implements #renderOn: method,
or a block with single argument.
(an argument passed is an Athens canvas).

I implement a simple view panning and zooming with mouse drag and mouse-wheel (correspondigly).

Example1: open scene view, by passing a simple rendering block.

| view |
view  := AthensSceneView new.

view scene: [:canvas |
	canvas surface clear:  Color black.
	canvas setPaint: Color red.
	canvas drawShape: (0@0 corner:120@100)
].
view openInWindow.

Example2: open scene view on imported SVG file (note you need 'Athens-SVG' package loaded):

| view |
view  := AthensSceneView new.
view scene: (AthensSVGConverter fromFile: 'lion.svg').

view openInWindow.
!

AthensSceneView subclass: #AthensInteractiveScene
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Morphic'!

!AthensInteractiveScene commentStamp: '<historical>' prior: 0!
i am like my superclass, can display any scene (object which understands #renderOn:  message)
and support free-form pan and zoom.

but in addition, i expecting that scene object understands following:

scene handleEvent: event in: sceneView at: position
 
This method should answer true if scene object wants to handle event by itself , preventing default handling of view (pan & zoom).
The point is coordinates of event in scene coordinate space (not screen coordinate space). The event is MorphicEvent (currently limited only to mouse move&button events).

Note that if scene view enters zooming or panning state, no events will be passed to scene object as long as state is active. 


!

!AthensInteractiveScene methodsFor: 'event handling' stamp: 'IgorStasenko 6/1/2013 05:13'!
eventHandledByScene: evt

	^  true == (scene handleEvent: evt in: self at: (self eventPositionInScene:evt))
	! !

!AthensInteractiveScene methodsFor: 'event handling' stamp: 'IgorStasenko 6/1/2013 05:08'!
eventPositionInScene: evt
	| pt | 
	
	pt := evt position - self topLeft.
	
	^ transform transform: pt! !

!AthensInteractiveScene methodsFor: 'event handling' stamp: 'IgorStasenko 6/1/2013 05:17'!
mouseDown: evt

	((self inState: #panning) or: [ self inState:#zooming ]) 
		ifFalse: [
			(self eventHandledByScene: evt) ifTrue: [ ^ self ]. 	
		].

	^ super mouseDown: evt! !

!AthensInteractiveScene methodsFor: 'event handling' stamp: 'IgorStasenko 6/1/2013 05:12'!
mouseMove: evt

	((self inState: #panning) or: [ self inState:#zooming ]) 
		ifFalse: [
			(self eventHandledByScene: evt) ifTrue: [ ^ self ]. 	
		].
	^ super mouseMove: evt! !

!AthensInteractiveScene methodsFor: 'event handling' stamp: 'IgorStasenko 6/1/2013 05:16'!
mouseUp: evt

	((self inState: #panning) or: [ self inState:#zooming ]) 
		ifFalse: [
			(self eventHandledByScene: evt) ifTrue: [ ^ self ]. 	
		].
		
	^ super mouseUp:evt ! !

!AthensSceneView methodsFor: 'colors' stamp: 'IgorStasenko 4/21/2013 15:01'!
backgroundColor
	^ Color gray! !

!AthensSceneView methodsFor: 'session management' stamp: 'IgorStasenko 4/21/2013 04:37'!
checkSession

	session == Smalltalk session ifFalse: [ 
		self initializeForNewSession
		 ]! !

!AthensSceneView methodsFor: 'session management' stamp: 'IgorStasenko 4/21/2013 04:40'!
createSurface
	| extent |
	extent := self extent asIntegerPoint.
	(scene respondsTo: #createSurface:) ifTrue: [
		surface := scene createSurface: extent.
	  ] ifFalse: [ surface := AthensCairoSurface extent: extent].
! !

!AthensSceneView methodsFor: 'drawing' stamp: 'IgorStasenko 12/21/2013 02:17'!
drawOn: canvas
	| px py |
	canvas fillRectangle: bounds color: self backgroundColor.
	self renderScene.

	surface displayOnMorphicCanvas: canvas at: bounds origin.

"	 translucentImage: surface asForm at: bounds origin."
	
	self showDebugInfo
		ifTrue: [ 
			px := transform x printShowingDecimalPlaces: 3.
			py := transform y printShowingDecimalPlaces: 3.
			canvas
				drawString: 'zoom: ' , (transform sx printShowingDecimalPlaces: 3) , ' pan: ' , px , ' @ ' , py
				at: bounds origin
				font: nil
				color: Color white ]! !

!AthensSceneView methodsFor: 'event handling' stamp: 'IgorStasenko 4/21/2013 05:02'!
handlesMouseDown: evt
	^ true! !

!AthensSceneView methodsFor: 'event handling' stamp: 'IgorStasenko 4/21/2013 05:13'!
handlesMouseOver: evt
	^ true! !

!AthensSceneView methodsFor: 'event handling' stamp: 'IgorStasenko 4/21/2013 06:01'!
handlesMouseWheel: event
	^ true! !

!AthensSceneView methodsFor: 'state tracking' stamp: 'IgorStasenko 4/21/2013 05:37'!
inState: stateName


	^ (state at: stateName ifAbsent: nil) notNil! !

!AthensSceneView methodsFor: 'state tracking' stamp: 'IgorStasenko 4/21/2013 05:45'!
inState: stateName do: aBlock


	^ (state at: stateName ifAbsent: nil) ifNotNil: aBlock ! !

!AthensSceneView methodsFor: 'initialize-release' stamp: 'IgorStasenko 8/8/2013 18:37'!
initialize

	super initialize.
	keepRefreshing := false.
	transform := AthensAffineTransform new.
	
	self hResizing: #spaceFill.
	self vResizing: #spaceFill.
	color := Color transparent.
	state := Dictionary new.
	self extent: self minimumExtent.
	
! !

!AthensSceneView methodsFor: 'session management' stamp: 'IgorStasenko 4/21/2013 04:40'!
initializeForNewSession

	self createSurface.
	session := Smalltalk session.
! !

!AthensSceneView methodsFor: 'accessing' stamp: 'IgorStasenko 6/3/2013 14:27'!
keepRefreshing: aBoolean
	keepRefreshing := aBoolean.
	
	keepRefreshing ifTrue: [  self startStepping ].! !

!AthensSceneView methodsFor: 'layout' stamp: 'IgorStasenko 4/21/2013 05:39'!
layoutChanged 
	"react on morph resize"
	super layoutChanged.
	surface ifNotNil: [  
		self extent asIntegerPoint ~= surface extent ifTrue: [ 
			self createSurface ]
	].
	! !

!AthensSceneView methodsFor: 'initialize-release' stamp: 'IgorStasenko 4/21/2013 06:50'!
minimumExtent
	^ 300@300! !

!AthensSceneView methodsFor: 'event handling' stamp: 'IgorStasenko 4/21/2013 06:12'!
mouseDown: evt

	| pos |
	
	pos := evt cursorPoint.
	
	"left button"
	evt redButtonPressed ifTrue: [ 
		self setState: #panning value: { transform x@transform y. pos }
		 ].
	
! !

!AthensSceneView methodsFor: 'event handling' stamp: 'IgorStasenko 4/21/2013 05:41'!
mouseEnter: evt

	self setState: #mouseIn! !

!AthensSceneView methodsFor: 'event handling' stamp: 'IgorStasenko 4/21/2013 05:44'!
mouseLeave: evt
	self 
		resetState: #mouseIn;
		resetState: #panning.! !

!AthensSceneView methodsFor: 'event handling' stamp: 'IgorStasenko 4/21/2013 06:15'!
mouseMove: evt

	self inState: #panning do: [ :startPanAndPos |
		| delta |
		delta := startPanAndPos first + ( evt cursorPoint - startPanAndPos second ).
		
		transform 
			x: delta x;
			y: delta y.
		self changed.
	] ! !

!AthensSceneView methodsFor: 'event handling' stamp: 'IgorStasenko 4/21/2013 05:44'!
mouseUp: evt

	"self halt."
	
	self resetState: #panning! !

!AthensSceneView methodsFor: 'event handling' stamp: 'SeanDeNigris 2/10/2015 13:41'!
mouseWheel: event


	"Handle a mouseWheel event."
	
	| center zoom  |
	
	center := transform inverseTransform: (event cursorPoint - bounds origin).

	zoom := 1.
	
	event isUp ifTrue: [ zoom := 1.25 ].
	event isDown ifTrue: [ zoom := 1/1.25 ].
	
	
	(self inState: #zooming) ifTrue: [   
		self updateZoom: zoom cursor: event cursorPoint.
	] ifFalse: [
		self startZooming: zoom center: center.
	]  ! !

!AthensSceneView methodsFor: 'drawing' stamp: 'IgorStasenko 8/12/2013 15:18'!
renderScene

	self checkSession.

	scene ifNotNil: [ 
		surface drawDuring: [ :acanvas | 
			surface clear.
			acanvas pathTransform loadAffineTransform: transform.
			scene isBlock
				ifTrue: [ scene cull: acanvas cull: self ]
				ifFalse: [ scene renderOn: acanvas ] ] ]! !

!AthensSceneView methodsFor: 'state tracking' stamp: 'IgorStasenko 4/21/2013 05:39'!
resetState: stateName

	state at: stateName put: nil.

	"receiver's state changed"
	
	self changed.! !

!AthensSceneView methodsFor: 'accessing' stamp: 'IgorStasenko 4/21/2013 05:51'!
scene
	^ scene! !

!AthensSceneView methodsFor: 'accessing' stamp: 'IgorStasenko 4/25/2013 01:19'!
scene: aScene
	
	scene := aScene.
	
	self changed.! !

!AthensSceneView methodsFor: 'state tracking' stamp: 'IgorStasenko 4/21/2013 05:39'!
setState: stateName

	state at: stateName put: true.

	"receiver's state changed"
	
	self changed.! !

!AthensSceneView methodsFor: 'state tracking' stamp: 'IgorStasenko 4/21/2013 05:39'!
setState: stateName value: aValue

	state at: stateName put: aValue.

	"receiver's state changed"
	
	self changed.! !

!AthensSceneView methodsFor: 'drawing' stamp: 'IgorStasenko 4/26/2013 14:46'!
showDebugInfo
	^ true ! !

!AthensSceneView methodsFor: 'zoom animation' stamp: 'IgorStasenko 4/22/2013 14:41'!
startZooming: zoom center: center
	| start end |

		
	start := Time millisecondClockValue.
	end := start + 250.

	self setState: #zooming value: {  center. transform copy. zoom. start. end}.

	self startStepping! !

!AthensSceneView methodsFor: 'zoom animation' stamp: 'IgorStasenko 6/1/2013 14:09'!
step

	self inState: #zooming do: [ :zoomState |
		| now start end center targetZoom factor |
		now := Time millisecondClockValue.
		start := zoomState at: 4.
		end := zoomState at: 5.
		
		(now between: start and:end) ifFalse: [ now := end.
			self resetState: #zooming.
			self stopStepping ].
		
		transform := (zoomState at: 2) copy.
		center := zoomState at: 1.
		
		factor :=  (now-start)/(end - start ).
		targetZoom := zoomState at: 3.
		targetZoom := 1* (1-factor) + (targetZoom * factor).
		
		transform
			translateBy: center;
			scaleBy:targetZoom;
			translateBy: center negated.
		
	].
	self changed.
! !

!AthensSceneView methodsFor: 'zoom animation' stamp: 'IgorStasenko 4/22/2013 14:22'!
stepTime
	^ 0! !

!AthensSceneView methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 6/3/2013 14:27'!
stopStepping
	keepRefreshing ifFalse: [
			^ super stopStepping ]! !

!AthensSceneView methodsFor: 'zoom animation' stamp: 'IgorStasenko 4/22/2013 15:01'!
updateZoom: zoom cursor: cursorPoint

	| zoomState targetZoom start end now fraction newCenter |
	zoomState := state at:#zooming.
		
	"change the target zoom and increase time a bit"
	targetZoom := zoomState at: 3.
	
	start := zoomState at: 4.
	end := zoomState at: 5.
		
	now := Time millisecondClockValue.
	
	(now > end) ifTrue: [  now := end ].
	
	 "interpolate linearly the target zoom factor over time start ... end"
	fraction := (now - start) / (end - start).
	
	"zoom goes from 1..target zoom"
	targetZoom := 1*(1-fraction) + (targetZoom * fraction).

	self step. "to update transform"
	zoomState at: 3 put: targetZoom * zoom.
	newCenter :=   transform inverseTransform: (cursorPoint - bounds origin).
	zoomState at: 1 put: newCenter.
	zoomState at: 2 put: transform copy.
	zoomState at: 4 put: now.
	zoomState at: 5 put: now + 250 .
! !

Morph subclass: #AthensWrapMorph
	instanceVariableNames: 'surface session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Morphic'!

!AthensWrapMorph commentStamp: '<historical>' prior: 0!
i making sure that all my submorphs will be drawn
using athens, but not balloon.

For that i use separate surface.!

!AthensWrapMorph methodsFor: 'accessing' stamp: 'IgorStasenko 9/6/2013 12:18'!
athensSurface
	self checkSurface.
	
	^ surface! !

!AthensWrapMorph methodsFor: 'surface management' stamp: 'IgorStasenko 9/6/2013 12:47'!
checkSurface

	session == Smalltalk session ifFalse: [ 
		self initializeForNewSession
		 ].
	
	bounds extent ~= surface extent ifTrue: [ 
		"recreate a surface if dimensions are changed"	
		self createSurface
	 ]
! !

!AthensWrapMorph methodsFor: 'surface management' stamp: 'IgorStasenko 9/6/2013 12:46'!
createSurface
	| extent |
	extent := bounds extent asIntegerPoint.
	surface := AthensCairoSurface extent: extent.
! !

!AthensWrapMorph methodsFor: 'drawing' stamp: 'IgorStasenko 8/26/2013 11:41'!
drawOn: aCanvas

	"do nothing"! !

!AthensWrapMorph methodsFor: 'drawing' stamp: 'IgorStasenko 8/26/2013 13:03'!
drawOnAthensCanvas: aCanvas

	"do nothing"! !

!AthensWrapMorph methodsFor: 'drawing' stamp: 'IgorStasenko 12/21/2013 02:18'!
fullDrawOn: aCanvas

	"Draw the full Morphic structure on the given Canvas"

	self visible ifFalse: [^ self].
	(aCanvas isVisible: self fullBounds) ifFalse:[^self].
	(self hasProperty: #errorOnDraw) ifTrue:[^self drawErrorOn: aCanvas].
	
	self checkSurface.

	[
		self render.
		surface displayOnMorphicCanvas: aCanvas at: bounds origin.
	
	] on: Error do: [:err |
		self setProperty: #errorOnDraw toValue: true.
		self setProperty: #drawError toValue: err freeze.
		^ self drawErrorOn: aCanvas
	]! !

!AthensWrapMorph methodsFor: 'drawing' stamp: 'IgorStasenko 8/26/2013 12:58'!
fullDrawOnAthensCanvas: anAthensCanvas 
	"Draw the full Morphic structure on the given Canvas"

	self visible ifFalse: [^ self].
	(anAthensCanvas isVisible: self fullBounds) ifFalse: [^self].
	(self hasProperty: #errorOnDraw) ifTrue: [^self "drawErrorOn: aCanvas" ].

	(anAthensCanvas isVisible: self bounds) ifTrue: [ anAthensCanvas draw: self ].
	
	submorphs reverseDo: [:m | anAthensCanvas fullDrawMorph: m ] 
! !

!AthensWrapMorph methodsFor: 'events handling' stamp: 'FernandoOlivero 9/2/2013 14:53'!
handlesMouseDown: evt
	^ true ! !

!AthensWrapMorph methodsFor: 'surface management' stamp: 'IgorStasenko 8/26/2013 11:44'!
initializeForNewSession

	self createSurface.
	session := Smalltalk session.
! !

!AthensWrapMorph methodsFor: 'drawing' stamp: 'JochenRick 1/10/2014 06:35'!
render
	surface drawDuring: [ :canvas |
		surface clear.
		canvas pathTransform restoreAfter: [ 
			"before rendering transform from a global coordinates (Morphic) to relative coordinates (Athens)"
			canvas pathTransform translateBy: self bounds origin negated.
			self fullDrawOnAthensCanvas: canvas.
		].
	].
! !

AthensWrapMorph subclass: #AthensWrapWorldMorph
	instanceVariableNames: 'worldmorph wrappedWorldDisplay'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Morphic'!

!AthensWrapWorldMorph commentStamp: 'NicolaiHess 1/4/2015 18:35' prior: 0!
An AthensWrapWorldMorph is a AthensWrapMorph holding a specialized Athens WorldMorph. This Athens WorldMorph draws the submorphs with an AthensCanvas.

Instance Variables
	worldmorph:		the Athens WorldMorph
	wrappedWorldDisplay:		a display wrapper
!

!AthensWrapWorldMorph methodsFor: 'initialization' stamp: 'NicolaiHess 1/4/2015 18:36'!
defaultExtent
	^ 400 @ 400! !

!AthensWrapWorldMorph methodsFor: 'drawing' stamp: 'NicolaiHess 2/17/2015 21:17'!
fullDrawOnAthensCanvas: anAthensCanvas 
	"Draw the full Morphic structure on the given Canvas"

	self visible ifFalse: [^ self].
	(anAthensCanvas isVisible: self fullBounds) ifFalse: [^self].
	(self hasProperty: #errorOnDraw) ifTrue: [^self "drawErrorOn: aCanvas" ].
	"draw athens wrap morph"
	(anAthensCanvas isVisible: self bounds) ifTrue: [ anAthensCanvas draw: self ].
	"draw wrapped world and its submorphs"
	submorphs reverseDo: [:m | anAthensCanvas fullDrawMorph: m ] ! !

!AthensWrapWorldMorph methodsFor: 'initialization' stamp: 'NicolaiHess 1/4/2015 18:36'!
initialize
	super initialize.
	self extent: self defaultExtent.
	self hResizing: #spaceFill.
	self vResizing: #spaceFill.
	wrappedWorldDisplay := AthensWrappedWorldDisplay on: self.
	worldmorph := AthensWorldMorph new.
	worldmorph color: Color orange.
	worldmorph addHand: HandMorph new.
	worldmorph display: wrappedWorldDisplay.
	self addMorph: worldmorph! !

!AthensWrapWorldMorph methodsFor: 'layout' stamp: 'NicolaiHess 1/4/2015 18:36'!
layoutChanged
	super layoutChanged.
	worldmorph ifNotNil:[:w |  w extent: self extent].! !

!AthensWrapWorldMorph methodsFor: 'drawing' stamp: 'NicolaiHess 1/6/2015 11:42'!
render
	surface
		drawDuring: [ :canvas | 
			surface clear.
			canvas pathTransform
				restoreAfter: [ 
					"before rendering transform from a global coordinates (Morphic) to relative coordinates (Athens)"
					canvas pathTransform translateBy: self bounds origin negated.
					self fullDrawOnAthensCanvas: canvas ] ]! !

!AthensWrapWorldMorph methodsFor: 'stepping' stamp: 'NicolaiHess 1/4/2015 18:36'!
step
	self changed! !

!AthensWrapWorldMorph methodsFor: 'stepping' stamp: 'NicolaiHess 1/4/2015 18:36'!
stepTime
	^ 20! !

!Morph methodsFor: '*Athens-Morphic' stamp: 'IgorStasenko 9/6/2013 12:12'!
athensSurface

	^ owner 
		ifNil: [  nil ]
		ifNotNil: [ owner athensSurface  ]! !

!Morph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/10/2015 18:30'!
drawClippedOnAthensCanvas: anAthensCanvas during: aBlock

	^ self clipSubmorphs 
		ifTrue: [ 
			 anAthensCanvas
				clipBy: (self clippingBounds intersect: anAthensCanvas clipRect ifNone:[^ self])
				during: aBlock ] 
		ifFalse: [ aBlock value ]
! !

!Morph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/7/2015 21:16'!
drawKeyboardFocusOnAthensCanvas: anAthensCanvas
	"Draw the keyboard focus indication."

	self focusIndicatorMorph
		drawOnAthensCanvas: anAthensCanvas! !

!Morph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/6/2015 19:32'!
drawOnAthensCanvas: anAthensCanvas 
	| border |
	border := self borderStyle. 
	anAthensCanvas setPaint: self fillStyle.
	anAthensCanvas drawShape: self bounds.
	
	"Fill the given rectangle."
	anAthensCanvas drawShape: (self bounds insetBy: border width).

	(anAthensCanvas setStrokePaint: border).
	anAthensCanvas drawShape: (self bounds insetBy: (border width /2 asFloat)).
	

	"aBorderStyle frameRectangle: aRectangle on: self "
	
	! !

!Morph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/7/2015 21:12'!
drawSubmorphsOnAthensCanvas: anAthensCanvas
	self
		drawClippedOnAthensCanvas: anAthensCanvas
		during: [ submorphs reverseDo: [ :m | anAthensCanvas fullDrawMorph: m ] ]! !

!Morph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/7/2015 21:14'!
fullDrawOnAthensCanvas: anAthensCanvas 
	"Draw the full Morphic structure on the given Canvas"

	self visible ifFalse: [^ self].
	(anAthensCanvas isVisible: self fullBounds) ifFalse: [^self].
	(self hasProperty: #errorOnDraw) ifTrue: [^self "drawErrorOn: aCanvas" ].

	(anAthensCanvas isVisible: self bounds) ifTrue: [ anAthensCanvas draw: self ].
	
	submorphs isEmpty ifTrue: [^self].

	self drawSubmorphsOnAthensCanvas: anAthensCanvas
! !

!Morph methodsFor: '*Athens-Morphic' stamp: 'IgorStasenko 11/7/2012 19:16'!
testAthensRender

| surf |
	surf := AthensCairoSurface extent: Display extent.

	surf drawDuring: [:can |
"		can pathTransform scaleBy: 0.5; rotateByDegrees: 30."
		self fullDrawOnAthensCanvas: can ].

	Display getCanvas drawImage: 	surf asForm  at: 0@0
! !

!Tab methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/7/2015 21:40'!
createClosedPolygonPathFrom: aPointsArray on: anAthensCanvas
	^ anAthensCanvas
		createPath: [ :builder | 
			aPointsArray ifEmpty: [ builder ].
			builder absolute.
			builder moveTo: aPointsArray first.
			aPointsArray allButFirstDo: [ :p | builder lineTo: p ].
			builder close.
			builder ]! !

!Tab methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/7/2015 21:38'!
createLinesPathFrom: borderVertices on: anAthensCanvas
	^ anAthensCanvas
		createPath: [ :builder | 
			builder absolute.
			borderVertices
				overlappingPairsDo: [ :start :end | 
					builder moveTo: start.
					builder lineTo: end ] ]! !

!Tab methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/7/2015 21:39'!
drawOnAthensCanvas: anAthensCanvas
	| borderVertices vertices path |
	borderVertices := self buildBorderVertices.
	vertices := self buildVerticesFrom: borderVertices.
	path := self createClosedPolygonPathFrom: vertices on: anAthensCanvas.
	anAthensCanvas setPaint: self backgroundColor.
	anAthensCanvas drawShape: path.
	path := self createLinesPathFrom: borderVertices on: anAthensCanvas.
	(anAthensCanvas setStrokePaint: self borderColor) width:1.
	anAthensCanvas drawShape: path.
! !

!Paragraph methodsFor: '*Athens-Morphic' stamp: 'IgorStasenko 10/8/2012 04:21'!
drawOnAthensCanvas: aCanvas bounds: aRectangle 
	"do nothing"! !

!PluggableTextFieldMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/15/2015 12:40'!
drawOnAthensCanvas: anAthensCanvas
	"Eventually draw the helpText"
	super drawOnAthensCanvas: anAthensCanvas.
	self ghostText
		ifNotEmpty: [
			self text isEmpty
				ifTrue: [| ghostBounds |
					ghostBounds := ((self scroller submorphBounds topLeft corner: self innerBounds bottomRight)
						translateBy: 0 @ (self scroller height - self theme textFont height / 2) rounded)
						insetBy: self textMarginsWidth @ 0.
					ghostBounds inspectOnce.
					anAthensCanvas 
						morphicDrawString: self ghostText
						in: ghostBounds
						font: self theme textFont
						color: Color lightGray]].
	entryCompletion 
		ifNotNil: [entryCompletion closeIfNotNeeded: self].! !

!InfiniteForm methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/6/2015 18:09'!
asAthensPaintOn: canvas
	^ (canvas surface createFormPaint: self asForm)
		repeat;
		origin: self origin;
		direction: self direction;
		yourself! !

!MenuItemMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 8/5/2014 10:39'!
drawBackgroundOnAthensCanvas: anAthensCanvas
	(isSelected and: [ isEnabled ])
		ifFalse: [ ^ self ].
	anAthensCanvas setPaint: self selectionFillStyle.
	anAthensCanvas drawShape: (0 @ 0 extent: self extent)! !

!MenuItemMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 8/5/2014 10:45'!
drawIconAndLabelOnAthensCanvas: anAthensCanvas
	anAthensCanvas pathTransform
		restoreAfter: [ 
			self drawIconOnAthensCanvas: anAthensCanvas.
			self drawLabelOnAthensCanvas: anAthensCanvas ]! !

!MenuItemMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 8/5/2014 10:41'!
drawIconOnAthensCanvas: anAthensCanvas
	self hasIcon
		ifFalse: [ ^ self ].
	anAthensCanvas setPaint: self iconForm.
	anAthensCanvas drawShape: (0 @ 0 extent: self iconForm extent).
	anAthensCanvas pathTransform translateX: self iconForm width + 2 Y: 0! !

!MenuItemMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 8/5/2014 10:43'!
drawLabelOnAthensCanvas: anAthensCanvas
	anAthensCanvas pathTransform translateX: 0 Y: self fontToUse getPreciseAscent.
			anAthensCanvas
				setPaint: self color;
				setFont: self fontToUse;
				drawString: contents.! !

!MenuItemMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 8/5/2014 10:51'!
drawOnAthensCanvas: anAthensCanvas
	| bnd |
	bnd := self bounds topLeft + self layoutInset.
	anAthensCanvas pathTransform
		restoreAfter: [ 
			anAthensCanvas pathTransform translateBy: bnd.
			self drawBackgroundOnAthensCanvas: anAthensCanvas.
			self drawIconAndLabelOnAthensCanvas: anAthensCanvas.
			self drawSubMenuMarkerOnAthensCanvas: anAthensCanvas ]! !

!MenuItemMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 8/5/2014 10:52'!
drawSubMenuMarkerOnAthensCanvas: anAthensCanvas
	self hasSubMenu
		ifFalse: [ ^ self ].
	anAthensCanvas pathTransform
		restoreAfter: [ 
			anAthensCanvas pathTransform translateX: self extent x - self subMenuMarker width Y: 0.
			anAthensCanvas pathTransform translateX: 0 Y: (self subMenuMarker height //2).
			anAthensCanvas setPaint: self subMenuMarker.
			anAthensCanvas drawShape: (0 @ 0 extent: self subMenuMarker extent) ]! !

!MenuMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/15/2015 13:30'!
drawOnAthensCanvas: anAthensCanvas
	"Draw the menu. Add keyboard-focus feedback if appropriate"

	super drawOnAthensCanvas: anAthensCanvas.
	(ActiveHand notNil and: [ ActiveHand keyboardFocus == self and: [ self rootMenu hasProperty: #hasUsedKeyboard ] ])
		ifTrue: [ 
			(anAthensCanvas setStrokePaint: self theme menuKeyboardFocusColor) width: self theme menuBorderWidth.
			anAthensCanvas drawShape: self innerBounds ]! !

!BorderStyle methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/6/2015 19:29'!
asAthensPaintOn: anAthensCanvas
	| stroke |
	stroke := self color asStrokePaintOn: anAthensCanvas.
	stroke width: self width.
	stroke joinMiter.
	^ stroke! !

WorldMorph subclass: #AthensWorldMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Morphic'!

!AthensWorldMorph commentStamp: 'NicolaiHess 1/4/2015 18:32' prior: 0!
An AthensWorldMorph is a special version of a WorldMorph.
It is within a real Morph in a real morphic world. The Morph it is in defines the DisplayScreen dimension. (see AthensWrappedWorldDisplay)!

!AthensWorldMorph methodsFor: 'world state' stamp: 'NicolaiHess 1/4/2015 18:33'!
display
	worldState display! !

!AthensWorldMorph methodsFor: 'world state' stamp: 'NicolaiHess 1/4/2015 18:33'!
display: aWrappedDisplay
	worldState display: aWrappedDisplay! !

!AthensWorldMorph methodsFor: 'initialization' stamp: 'NicolaiHess 1/4/2015 18:33'!
initialize
	super initialize.
	worldState := AthensWorldState new! !

!AthensWorldMorph methodsFor: 'world state' stamp: 'NicolaiHess 1/4/2015 18:33'!
restoreMorphicDisplay
"overwritten for using its own display dimension"
	DisplayScreen startUp.

	self
		extent: self display extent;
		viewBox: self display boundingBox;
		handsDo: [:h | h visible: true; showTemporaryCursor: nil];
		resizeBackgroundMorph;
		restoreMainDockingBarDisplay;
		fullRepaintNeeded.
		
	self defer: [Cursor normal show].
		
	self layoutChanged.
! !

!AthensWorldMorph methodsFor: 'world state' stamp: 'NicolaiHess 1/4/2015 18:33'!
viewBox: newViewBox
	"I am now displayed within newViewBox; react."

	(self viewBox isNil or: [ self viewBox extent ~= newViewBox extent ])
		ifTrue: [ worldState canvas: nil ].
	worldState viewBox: newViewBox.
	super viewBox: newViewBox.
	worldState handsDo: [ :hand | hand releaseKeyboardFocus ].
	self fullRepaintNeeded! !

!CheckboxMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/7/2015 21:25'!
drawSubmorphsOnAthensCanvas: anAthensCanvas
	super 	drawSubmorphsOnAthensCanvas: anAthensCanvas.
	self hasKeyboardFocus ifTrue: [self drawKeyboardFocusOnAthensCanvas: anAthensCanvas].! !

AthensPaint subclass: #AthensBorderStrokePaint
	instanceVariableNames: 'topLeftPaint bottomRightPaint width'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Morphic'!

!AthensBorderStrokePaint commentStamp: '<historical>' prior: 0!
i am used to convert 
ComplexBorder (raised/inset) into paint, used in Morphic.!

!AthensBorderStrokePaint class methodsFor: 'instance creation' stamp: 'NicolaiHess 1/6/2015 22:27'!
topLeftColor: aColor1 bottomRightColor: aColor2 width: aNumber on: anAthensCanvas
	^ self new
		topLeftPaint: (aColor1 asAthensPaintOn: anAthensCanvas);
		bottomRightPaint: (aColor2 asAthensPaintOn: anAthensCanvas);
		width: aNumber;
		yourself! !

!AthensBorderStrokePaint methodsFor: 'drawing' stamp: 'NicolaiHess 1/6/2015 22:26'!
asStrokePaintOn: aCanvas
	"receiver is stroke paint already"
	^ self! !

!AthensBorderStrokePaint methodsFor: 'drawing' stamp: 'NicolaiHess 1/6/2015 22:27'!
athensFillRectangle: aRect on: anAthensCanvas
	"This is a terminal method in rendering dispatch scheme
	canvas->shape->paint. See AthensCanvas>>fillShape: "

	topLeftPaint
		athensFillPath: (self pathForTopLeftBorderOf: (aRect expandBy: width/2.0) on: anAthensCanvas)
		on: anAthensCanvas.
	bottomRightPaint
		athensFillPath: (self pathForBottomRightBorderOf: (aRect expandBy: width/2.0) on: anAthensCanvas)
		on: anAthensCanvas! !

!AthensBorderStrokePaint methodsFor: 'accessing' stamp: 'NicolaiHess 1/6/2015 22:26'!
bottomRightPaint: anAthensPaint
	bottomRightPaint := anAthensPaint! !

!AthensBorderStrokePaint methodsFor: 'private' stamp: 'NicolaiHess 1/6/2015 22:26'!
pathForBottomRightBorderOf: aRectangle on: anAthensCanvas
	^ anAthensCanvas
		createPath: [ :builder | 
			builder
				absolute;
				moveToX: aRectangle left Y: aRectangle bottom;
				lineToX: aRectangle right Y: aRectangle bottom;
				lineToX: aRectangle right Y: aRectangle top;
				lineToX: aRectangle right - width Y: aRectangle top + width;
				lineToX: aRectangle right - width Y: aRectangle bottom - width;
				lineToX: aRectangle left + width Y: aRectangle bottom - width;
				close ]! !

!AthensBorderStrokePaint methodsFor: 'private' stamp: 'NicolaiHess 1/6/2015 22:26'!
pathForTopLeftBorderOf: aRectangle on: anAthensCanvas
	^ anAthensCanvas
		createPath: [ :builder | 
			builder
				absolute;
				moveToX: aRectangle left Y: aRectangle bottom;
				lineToX: aRectangle left Y: aRectangle top;
				lineToX: aRectangle right Y: aRectangle top;
				lineToX: aRectangle right - width Y: aRectangle top + width;
				lineToX: aRectangle left + width Y: aRectangle top + width;
				lineToX: aRectangle left + width Y: aRectangle bottom - width;
				close ]! !

!AthensBorderStrokePaint methodsFor: 'accessing' stamp: 'NicolaiHess 1/6/2015 22:26'!
topLeftPaint: anAthensPaint
	topLeftPaint := anAthensPaint! !

!AthensBorderStrokePaint methodsFor: 'accessing' stamp: 'NicolaiHess 1/6/2015 22:26'!
width
	^ width! !

!AthensBorderStrokePaint methodsFor: 'accessing' stamp: 'NicolaiHess 1/6/2015 22:26'!
width: aWidth
	width:= aWidth! !

AthensPaint subclass: #AthensCompositePaint
	instanceVariableNames: 'paints'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Morphic'!

!AthensCompositePaint commentStamp: '<historical>' prior: 0!
i am used to convert CompositeFillStyle into paint, used in Morphic.!

!AthensCompositePaint methodsFor: 'drawing' stamp: 'IgorStasenko 8/30/2013 16:32'!
athensFillPath: athensPath on: anAthensCanvas 

	"This is a terminal method in rendering dispatch scheme
	canvas->shape->paint. See AthensCanvas>>fillShape: "

	paints do: [:paint | 
				paint athensFillPath: athensPath on: anAthensCanvas ]
! !

!AthensCompositePaint methodsFor: 'drawing' stamp: 'IgorStasenko 8/30/2013 16:35'!
athensFillRectangle: aRect on: anAthensCanvas
	"This is a terminal method in rendering dispatch scheme
	canvas->shape->paint. See AthensCanvas>>fillShape: "
	

	paints do: [:paint | 
				paint athensFillRectangle: aRect on: anAthensCanvas ]
! !

!AthensCompositePaint methodsFor: 'initialize-release' stamp: 'IgorStasenko 10/9/2012 19:07'!
fromFillStyles: fillStyles on: aCanvas

	paints := fillStyles collect: [:each | each asAthensPaintOn: aCanvas ]! !

AthensPaint subclass: #AthensCompositeStrokePaint
	instanceVariableNames: 'strokePaints withInset'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Morphic'!

!AthensCompositeStrokePaint commentStamp: '<historical>' prior: 0!
i am used to convert CompositeBorderStyle into paint, used in Morphic.
The "withInset" attribute defines whether each sub stroke  outlines
a smaller rectangle. If not, all strokes draw the same rectangle.
(useful for multi-color-dashes)!

!AthensCompositeStrokePaint methodsFor: 'converting' stamp: 'NicolaiHess 1/6/2015 18:33'!
asStrokePaintOn: aCanvas
	"receiver is stroke paint already"
	^ self! !

!AthensCompositeStrokePaint methodsFor: 'drawing' stamp: 'NicolaiHess 1/15/2015 10:23'!
athensFillRectangle: aRect on: anAthensCanvas
	"This is a terminal method in rendering dispatch scheme
	canvas->shape->paint. See AthensCanvas>>fillShape: "

	| r |
	r := aRect.
	strokePaints
		do: [ :paint | 
			paint athensFillRectangle: r on: anAthensCanvas.
			withInset ifTrue:[r := r insetBy: paint width ]]! !

!AthensCompositeStrokePaint methodsFor: 'private' stamp: 'NicolaiHess 1/6/2015 18:32'!
fromBorderStyles: borders on: anAthensCanvas
	strokePaints := borders collect: [:each | (each asAthensPaintOn: anAthensCanvas ) asStrokePaintOn: anAthensCanvas]! !

!AthensCompositeStrokePaint methodsFor: 'private' stamp: 'NicolaiHess 1/15/2015 11:51'!
fromDashedBorderStyle: border on: anAthensCanvas
	| allDashLength offset width |
	width := border width.
	allDashLength := border dashLengths detectSum: #yourself.
	offset := 0.
	strokePaints := border dashColors
		collectWithIndex: [ :c :i | 
			| stroke dashLength |
			dashLength := border dashLengths at: i.
			stroke := c asStrokePaintOn: anAthensCanvas.
			stroke width: width.
			stroke capButt.
			stroke
				dashes:
					{(dashLength * width).
					((allDashLength - dashLength) * width)}
				offset: (allDashLength - offset) * width.
			offset := offset + dashLength.
			stroke ].
	self withInset: false! !

!AthensCompositeStrokePaint methodsFor: 'drawing' stamp: 'NicolaiHess 1/15/2015 10:20'!
initialize

	super initialize.

	strokePaints := nil.
	withInset := true.! !

!AthensCompositeStrokePaint methodsFor: 'accessing' stamp: 'NicolaiHess 1/15/2015 10:20'!
withInset: aBoolean
	withInset := aBoolean! !

AthensPaint subclass: #AthensMorphicGradientPaint
	instanceVariableNames: 'gradient'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Morphic'!

!AthensMorphicGradientPaint commentStamp: '<historical>' prior: 0!
i am used to convert gradient fill style(s) used in Morphic to appropriate paint(s) in Athens.!

!AthensMorphicGradientPaint methodsFor: 'rendering' stamp: 'NicolaiHess 8/7/2014 11:06'!
athensFillPath: aPath on: canvas
	"This is a terminal method in rendering dispatch scheme
	canvas->shape->paint. See AthensCanvas>>fillShape: "
	
	| fill |
	fill := self convertGradientToPaintOn: canvas.
	fill athensFillPath: aPath on: canvas.
! !

!AthensMorphicGradientPaint methodsFor: 'rendering' stamp: 'NicolaiHess 8/7/2014 11:06'!
athensFillRectangle: aRect on: canvas
	"This is a terminal method in rendering dispatch scheme
	canvas->shape->paint. See AthensCanvas>>fillShape: "
	
	| fill |
	fill := self convertGradientToPaintOn: canvas.
	fill athensFillRectangle: aRect on: canvas.
! !

!AthensMorphicGradientPaint methodsFor: 'private' stamp: 'NicolaiHess 8/7/2014 11:05'!
convertGradientToPaintOn: aCanvas
	"This is a terminal method in rendering dispatch scheme
	canvas->shape->paint. See AthensCanvas>>fillShape: "
	
	^ gradient radial ifTrue: [
		aCanvas surface
			createRadialGradient: gradient colorRamp
			center:  (aCanvas pathTransform inverseTransform: gradient origin )
			radius: gradient direction y ]
	ifFalse: [
		aCanvas surface
			createLinearGradient: gradient colorRamp
			start: (aCanvas pathTransform inverseTransform: gradient origin)
			stop: (aCanvas pathTransform inverseTransform: gradient origin + gradient direction ) ].

! !

!AthensMorphicGradientPaint methodsFor: 'accessing' stamp: 'IgorStasenko 10/11/2012 16:14'!
gradient: aGradient
	gradient := aGradient! !

!DashedBorder methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/15/2015 10:44'!
asAthensPaintOn: anAthensCanvas
	^ AthensCompositeStrokePaint new fromDashedBorderStyle: self on: anAthensCanvas! !

!PluggableButtonMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/10/2015 17:40'!
drawSubmorphsOnAthensCanvas: anAthensCanvas
	super 	drawSubmorphsOnAthensCanvas: anAthensCanvas.
	self hasKeyboardFocus ifTrue: [self drawKeyboardFocusOnAthensCanvas: anAthensCanvas].
! !

!TextMorph methodsFor: '*Athens-Morphic' stamp: 'IgorStasenko 3/8/2012 14:02'!
drawOnAthensCanvas: aCanvas

	"Draw the receiver on a canvas.
	Draw keyboard focus if appropriate."

	| fauxBounds |

	self setDefaultContentsIfNil.
	super drawOnAthensCanvas: aCanvas.  "Border and background if any"
	
	"nothing to draw?"
	(self startingIndex > text size)
		ifTrue: [ ^ self ].

	"Hack here:  The canvas expects bounds to carry the location of the text, but we also need to communicate clipping."
	fauxBounds := self bounds topLeft corner: self innerBounds bottomRight.
	
	self paragraph drawOnAthensCanvas: aCanvas bounds: fauxBounds.
	
	! !

!NewListRenderer methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/10/2015 17:47'!
drawSubmorphsOnAthensCanvas: anAthensCanvas
	"like drawSubmorphsOn:, do nothing"! !

!ImageMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/6/2015 19:46'!
drawOnAthensCanvas: aCanvas
	
	| cached style|
	
	cached := aCanvas cacheAt: image ifAbsentPut: [
		image asAthensPaintOn: aCanvas.
		].
	
	aCanvas setPaint: cached.
		
		aCanvas paintTransform restoreAfter: [
			aCanvas paintTransform translateBy: self innerBounds origin "negated".
		
			aCanvas drawShape: self innerBounds.
		].
	(style := self borderStyle) ifNotNil: [
		aCanvas setStrokePaint: style.
		aCanvas drawShape: bounds]	
! !

!AthensCanvas methodsFor: '*Athens-Morphic' stamp: 'IgorStasenko 4/21/2011 14:27'!
fullDrawMorph: aMorph

"Hook method for potential other canvases. In the core, this method looks supefluous but PostscriptCanvases and other canvases can specialized it for Morph rendering. Therefore it should not be merged with fullDraw:."

	aMorph fullDrawOnAthensCanvas: self! !

!AthensCanvas methodsFor: '*Athens-Morphic' stamp: 'IgorStasenko 10/11/2012 15:25'!
morphicDrawString: string at: position font: aFont color: aColor

	self pathTransform restoreAfter: [
		self pathTransform translateX: position x Y: position y + aFont getPreciseAscent.
		self 
			setFont: aFont;
			setPaint: aColor;
			drawString: string.
	]
! !

!AthensCanvas methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/23/2015 13:58'!
morphicDrawString: string in: rect font: aFont color: aColor
	self
		clipBy: rect
		during: [ 
			self pathTransform
				restoreAfter: [ 
					self pathTransform translateX: rect left Y: rect top.
					self setFont: aFont.
					self pathTransform translateX: 0 Y: aFont getPreciseAscent.
					self
						setPaint: aColor;
						drawString: string ] ]! !

!SimpleBorder methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/6/2015 22:28'!
asAthensPaintOn: anAthensCanvas
	^ self topLeftColor = self bottomRightColor
		ifTrue: [ super asAthensPaintOn: anAthensCanvas ]
		ifFalse: [ self asTwoColorStrokePaintOn: anAthensCanvas ]! !

!SimpleBorder methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/6/2015 22:27'!
asTwoColorStrokePaintOn: anAthensCanvas
	^ AthensBorderStrokePaint
		topLeftColor: self topLeftColor
		bottomRightColor: self bottomRightColor
		width: self width
		on: anAthensCanvas! !

!AlphaInfiniteForm methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/6/2015 18:25'!
asAthensPaintOn: canvas
	^ (canvas surface createFormPaint: self asForm)
		repeat;
		origin: self origin;
		direction: self direction;
		extent: self extent;
		yourself! !

!TransformMorph methodsFor: '*Athens-Morphic' stamp: 'IgorStasenko 10/7/2012 23:18'!
clipDuring: aBlock canvas: aCanvas 

	self clipSubmorphs 
		ifTrue: [ 
			aCanvas
				clipBy: self clippingBounds 
				during: aBlock ] 
		ifFalse: [ ^ aBlock value ]
! !

!TransformMorph methodsFor: '*Athens-Morphic' stamp: 'IgorStasenko 10/7/2012 23:22'!
drawClippedOn: aCanvas during: aBlock

	^ self clipSubmorphs 
		ifTrue: [ 
			 aCanvas
				clipBy: self clippingBounds 
				during: aBlock ] 
		ifFalse: [ aBlock value ]
! !

!TransformMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/22/2015 22:47'!
drawSubmorphsOnAthensCanvas: anAthensCanvas
	| clip |
	clip := self clippingBounds intersect: anAthensCanvas clipRect.
	anAthensCanvas
		transformBy: self transform
		withClip: clip
		in: self innerBounds
		during: [ :c | submorphs do: [ :m | m fullDrawOnAthensCanvas: c ] ]! !

!AthensCairoSurface methodsFor: '*Athens-Morphic' stamp: 'IgorStasenko 12/21/2013 02:17'!
displayOnMorphicCanvas: canvas at: aPoint
	
	canvas
		image: self asForm
		at: aPoint
		sourceRect: (0@0 extent: self extent )
		rule: 34.
! !

!AthensBezier3Scene class methodsFor: '*Athens-Morphic' stamp: 'IgorStasenko 4/26/2013 17:53'!
openViewOn: aControlPointsArray
	
	^ AthensSceneView new
		scene: 	(self on: aControlPointsArray );
		openInWindow! !

TextMorphForEditView subclass: #AthensTextMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Morphic'!

!AthensTextMorph commentStamp: '<historical>' prior: 0!
this class is subject of changes/removal. do not use it.!

!AthensTextMorph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/21/2011 12:42'!
contents: stringOrText wrappedTo: width
	"Accept new text contents.  Lay it out, wrapping to width.
	Then fit my height to the result."
	wrapFlag := true.
	super extent: width truncated@self height.
	self newContents: stringOrText! !

!AthensTextMorph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/21/2011 12:23'!
createParagraph
	| newParagraph |
	self setDefaultContentsIfNil.

	"...Code here to recreate the paragraph..."
	newParagraph  := AthensParagraph new.
	
	newParagraph
		compose: text
		style: textStyle copy
		in: self container.
		
		
	wrapFlag ifFalse:
		["Was given huge container at first... now adjust"
		newParagraph adjustRightX].
	
	newParagraph focused: (self currentHand keyboardFocus == self).
	
	paragraph := newParagraph.
	self fit.

	^ paragraph! !

!AthensTextMorph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/21/2011 12:18'!
paragraph
	^ paragraph ifNil: [ paragraph := self createParagraph ].
! !

!AthensTextMorph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/21/2011 13:02'!
selectionColor: aColor
	selectionColor := aColor.
	! !

!NewList methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/10/2015 18:26'!
drawOnAthensCanvas: anAthensCanvas
	anAthensCanvas setPaint: self drawColor.
	anAthensCanvas drawShape: self fullBounds! !

!ThreePhaseButtonMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/10/2015 18:01'!
drawOnAthensCanvas: anAthensCanvas
	| paint |
	paint := offImage.
	
	state == #pressed
		ifTrue: [ paint := pressedImage ].
	state == #on
		ifTrue: [ paint := image ].
	paint ifNil: [ ^ self ].
	
	anAthensCanvas setPaint: paint.
	anAthensCanvas paintTransform
		restoreAfter: [ 
			anAthensCanvas paintTransform translateBy: self innerBounds origin.	
			anAthensCanvas drawShape: self innerBounds ]! !

!LazyMorphListMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/15/2015 21:42'!
athensDisplay: item atRow: row on: aCanvas
	"Display the given item at the given row on the given canvas."

	row = self mouseDownRow
		ifTrue: [ 
			(aCanvas setStrokePaint: (self colorForRow: row))
				width: 1;
				dashes: {1} offset: 0.
			aCanvas drawShape: (self selectionFrameForRow: row) ]! !

!LazyMorphListMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/22/2015 22:53'!
drawSubmorphsOnAthensCanvas: anAthensCanvas 
	"Display submorphs back to front"

	| drawBlock|
	submorphs isEmpty ifTrue: [^self].
	drawBlock := [:canvas | | i |
		(self topVisibleRowForCanvas: anAthensCanvas) to: (self bottomVisibleRowForCanvas: anAthensCanvas) do: [ :row |
			i := self item: row.
			canvas fullDrawMorph: i]].
	self clipSubmorphs 
		ifTrue: [anAthensCanvas clipBy: (anAthensCanvas clipRect intersect: self clippingBounds ifNone: [ ^ self ]) during: drawBlock]
		ifFalse: [drawBlock value: anAthensCanvas]! !

!AColorSelectorMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/6/2015 18:11'!
drawOnAthensCanvas: anAthensCanvas
	anAthensCanvas setPaint: (InfiniteForm with: self hatchForm).
	anAthensCanvas drawShape: self innerBounds.
	super drawOnAthensCanvas: anAthensCanvas! !

!CompositeFillStyle methodsFor: '*Athens-Morphic' stamp: 'IgorStasenko 10/9/2012 19:10'!
asAthensPaintOn: aCanvas 
	^ AthensCompositePaint new fromFillStyles: fillStyles on: aCanvas! !

!AthensCairoCanvas methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/22/2015 22:46'!
transformBy: aDisplayTransform withClip: aClipRectangle in: aSubRectangle during: aDrawBlock
	self
		clipBy: currentClipRect
		during: [ 
			| subClip oldClip |
			subClip := aDisplayTransform globalBoundsToLocal: aClipRectangle.
			oldClip := currentClipRect.
			currentClipRect := aDisplayTransform globalBoundsToLocal: self clipRect.
			pathTransform
				restoreAfter: [ 
					pathTransform translateBy: aDisplayTransform offset negated.
					pathTransform rotateByRadians: aDisplayTransform angle negated.
					pathTransform scaleBy: aDisplayTransform scale.
					self clipBy: subClip during: [ aDrawBlock value: self ] ].
			currentClipRect := oldClip ]! !

!DropListMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/7/2015 21:25'!
drawSubmorphsOnAthensCanvas: anAthensCanvas
	super 	drawSubmorphsOnAthensCanvas: anAthensCanvas.
	self hasKeyboardFocus ifTrue: [self drawKeyboardFocusOnAthensCanvas: anAthensCanvas].! !

!IndentingListItemMorph methodsFor: '*Athens-Morphic' stamp: 'IgorStasenko 10/11/2012 14:56'!
drawOnAthensCanvas: aCanvas 
	| tRect sRect columnScanner colorToUse columnLeft |
	tRect := self toggleRectangle.
	sRect := bounds withLeft: tRect right + 4.
"	self 
		athensDrawToggleOn: aCanvas
		in: tRect.
"		
	colorToUse := complexContents preferredColor ifNil: [ color ].
	icon isNil ifFalse: 
		[ "aCanvas 
			translucentImage: icon
			at: sRect left @ (self top + ((self height - icon height) // 2))."
		sRect := sRect left: sRect left + icon width + 2 ].
	(container columns isNil or: [ (contents asString indexOf: Character tab) = 0 ]) 
		ifTrue: 
			[ sRect := sRect top: (sRect top + sRect bottom - self fontToUse height) // 2.
			"contents
				treeRenderOn: aCanvas 
				bounds: sRect 
				color: colorToUse 
				font: self 
				fontToUse 
				from: self" ]
		ifFalse: 
			[ columnLeft := sRect left.
			columnScanner := contents asString readStream.
			container columns do: 
				[ :width | | columnData columnRect | 
				columnRect := columnLeft @ sRect top extent: width @ sRect height.
				columnData := columnScanner upTo: Character tab.
				columnData isEmpty ifFalse: 
					[ aCanvas 
						morphicDrawString: columnData
						in: columnRect
						font: self fontToUse
						color: colorToUse ].
				columnLeft := columnRect right + 5 ] ]! !

!ColorFillStyle methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/6/2015 18:30'!
asAthensPaintOn: anAthensCanvas
	^ (anAthensCanvas surface createSolidColorPaint: color)
		origin: origin;
		extent: extent;
		yourself! !

!TransformationMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/10/2015 18:15'!
drawOnAthensCanvas: anAthensCanvas
	submorphs isEmpty ifTrue: [super drawOnAthensCanvas: anAthensCanvas]! !

WorldState subclass: #AthensWorldState
	instanceVariableNames: 'display'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Morphic'!

!AthensWorldState commentStamp: 'NicolaiHess 1/4/2015 18:31' prior: 0!
An AthensWorldState is a subclass of WorldState, that uses a special (fake) Display based on a real Morph. This is used to simulate a WorldMorph that actually resides in another Morph. This display adapts the owner dimensions as the "DisplayScreen"-Dimension.

!

!AthensWorldState methodsFor: 'accessing' stamp: 'NicolaiHess 1/4/2015 18:32'!
display
	^ display! !

!AthensWorldState methodsFor: 'accessing' stamp: 'NicolaiHess 1/4/2015 18:32'!
display: aDisplay
	display := aDisplay! !

!AthensWorldState methodsFor: 'update cycle' stamp: 'NicolaiHess 1/4/2015 18:32'!
doDeferredUpdatingFor: aWorld
	"overwritten from WorldState to use ower own display"

	(self display deferUpdates: true) ifNil: [ ^ false ].
	(canvas notNil and: [ canvas form == Display ])
		ifFalse: [ 
			aWorld viewBox: self display boundingBox.	"do first since it may clear canvas"
			self canvas: (self display getCanvas copyClipRect: self display boundingBox) ].
	^ true! !

!CompositeBorder methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/6/2015 18:33'!
asAthensPaintOn: anAthensCanvas
	^ AthensCompositeStrokePaint new fromBorderStyles: borders on: anAthensCanvas! !

!ScrollPane methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/7/2015 21:17'!
drawSubmorphsOnAthensCanvas: anAthensCanvas
	super 	drawSubmorphsOnAthensCanvas: anAthensCanvas.
	self hasKeyboardFocus ifTrue: [self drawKeyboardFocusOnAthensCanvas: anAthensCanvas].! !

!LazyListMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/15/2015 21:43'!
athensDisplay: item atRow: row on: aCanvas
	"Display the given item at the given row on the given
	canvas."

	| c backgroundColor drawBounds |
	item inspectOnce.
	drawBounds := self drawBoundsForRow: row.
	c := self colorForRow: row.
	backgroundColor := self backgroundColorForRow: row.
	aCanvas
		morphicDrawString: item asString
		at: drawBounds topLeft
		font: self font
		color: c.
	row = self mouseDownRow
		ifTrue: [ 
			(aCanvas setStrokePaint: c)
				width: 1;
				dashes: {1} offset: 0.
			aCanvas drawShape: (self selectionFrameForRow: row) ]! !

!LazyListMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/6/2015 18:12'!
athensDrawBackgroundForRow: row on: aCanvas color: aColor

	aCanvas setPaint: aColor.
	aCanvas drawShape: (self selectionFrameForRow: row)! !

!LazyListMorph methodsFor: '*Athens-Morphic' stamp: 'IgorStasenko 10/9/2012 19:04'!
athensDrawBackgroundForSearchedRow: row on: aCanvas 

	aCanvas setPaint: listSource  secondarySelectionColor.
	aCanvas drawShape: (self selectionFrameForRow: row)! !

!LazyListMorph methodsFor: '*Athens-Morphic' stamp: 'IgorStasenko 10/9/2012 19:03'!
athensDrawBackgroundForSelectedRow: row on: aCanvas 

	aCanvas setPaint: listSource selectionColorToUse.
	aCanvas drawShape: (self selectionFrameForRow: row)! !

!LazyListMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/15/2015 21:46'!
athensDrawSelectionOn: anAthensCanvas 
	
	selectedRow ifNil: [ ^self ].
	selectedRow = 0 ifTrue: [ ^self ].
	self athensDrawBackgroundForSelectedRow: selectedRow on: anAthensCanvas 
! !

!LazyListMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/15/2015 21:25'!
athensHighlightPotentialDropRow: row on: anAthensCanvas
	| drawBounds |
	drawBounds := self drawBoundsForRow: row.
	drawBounds := drawBounds intersect: self bounds ifNone: [ ^ self ].
	anAthensCanvas setStrokePaint: Color blue.
	anAthensCanvas drawShape: drawBounds! !

!LazyListMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/15/2015 21:39'!
drawOnAthensCanvas: aCanvas
	listItems isEmpty
		ifTrue: [^ self].
	"self drawSelectionOn: aCanvas."
	1 to: listItems size do: [:row |
		(listSource itemSelectedAmongMultiple: row)
			ifTrue: [ self athensDrawBackgroundForSelectedRow: row on: aCanvas]
			ifFalse: [
				(listSource searchedElement = row)
					ifTrue: [ self athensDrawBackgroundForSearchedRow: row on: aCanvas]
					ifFalse: [
						(listSource backgroundColorFor: row)
							ifNotNil: [:col |
								self athensDrawBackgroundForRow: row on: aCanvas color: col ]]].
					
			selectedRow = row ifTrue: [ self athensDrawSelectionOn: aCanvas ].
			(listSource separatorAfterARow: row) ifTrue: [ self athensDrawSeparatorAfter: row on: aCanvas ].
			self
				athensDisplay: (self item: row)
				atRow: row
				on: aCanvas].
		
	listSource potentialDropRow > 0
		ifTrue: [self athensHighlightPotentialDropRow: listSource potentialDropRow on: aCanvas]! !

Object subclass: #AthensParagraph
	instanceVariableNames: 'text textStyle containerRect lines positionWhenComposed selectionStart selectionStop'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Morphic'!

!AthensParagraph commentStamp: '<historical>' prior: 0!
this class is subject of changes/removal. do not use it.!

!AthensParagraph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 8/31/2012 01:31'!
caretRect ^ (0@0 corner: 0@0)  ! !

!AthensParagraph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/21/2011 16:08'!
characterBlockForIndex: index 

		^ self defaultCharacterBlock! !

!AthensParagraph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/21/2011 12:22'!
compose: aText style: aTextStyle from: startingIndex in: aRectangle 
	text := aText.
	textStyle := aTextStyle
	
	self shouldBeImplemented.! !

!AthensParagraph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/21/2011 12:50'!
compose: aText style: aTextStyle in: rect 

	text := aText.
	textStyle := aTextStyle.
	containerRect := rect.
	lines := (AthensTextComposer scan: aText for: AthensTextScanner new) lines.! !

!AthensParagraph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/21/2011 13:04'!
defaultCharacterBlock
	^ (CharacterBlock new stringIndex: 1 text: text
			topLeft: 0@0 extent: 0 @ 0)
		textLine: lines first! !

!AthensParagraph methodsFor: 'drawing' stamp: 'IgorStasenko 10/7/2012 23:32'!
drawOnAthensCanvas: canvas bounds: aRectangle 

	| renderer |
	"self halt."
"	canvas
		clipBy: aRectangle 
		during: [  
"			renderer := AthensTextRenderer new.
			renderer render: text lines: lines on: canvas.
"		].
"! !

!AthensParagraph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/21/2011 12:48'!
extent
	^ containerRect width @ lines last bottom! !

!AthensParagraph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/21/2011 13:07'!
extraSelectionChanged
"
	refreshExtraSelection := true
"

	self flag: #foo! !

!AthensParagraph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/21/2011 13:06'!
findReplaceSelectionRegex: aRegex
"	findReplaceSelectionRegex := aRegex. "
	self flag: #foo
! !

!AthensParagraph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 8/31/2012 01:28'!
focused ^ false! !

!AthensParagraph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/21/2011 12:45'!
focused: bool

	"why we should care?"
	
	"self flag: #ooo"! !

!AthensParagraph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/21/2011 13:00'!
positionWhenComposed: pos
	positionWhenComposed := pos! !

!AthensParagraph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/21/2011 13:06'!
secondarySelection: aSubString
"	secondarySelection := aSubString."
	self flag: #foo.! !

!AthensParagraph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/21/2011 13:02'!
selectionColor: anUndefinedObject 
	self shouldBeImplemented.! !

!AthensParagraph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/21/2011 13:08'!
selectionRects
	^ #()! !

!AthensParagraph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/21/2011 13:09'!
selectionStart: startBlock selectionStop: stopBlock
	selectionStart := startBlock.
	selectionStop := stopBlock.! !

!AthensParagraph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 8/31/2012 01:28'!
showCared ! !

!AthensParagraph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 8/31/2012 01:29'!
showCaret  ! !

!AthensParagraph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 8/31/2012 01:29'!
showCaret: abool  ! !

!AthensParagraph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/21/2011 13:05'!
text
	^ text! !

!AthensParagraph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/21/2011 12:20'!
textOwner: anUndefinedObject 
	self shouldBeImplemented.! !

Object subclass: #AthensWrappedWorldDisplay
	instanceVariableNames: 'realDisplay wrapMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Morphic'!

!AthensWrappedWorldDisplay commentStamp: 'NicolaiHess 1/4/2015 18:29' prior: 0!
An AthensWrappedWorldDisplay acts as a DisplayScreen
for a wrapped WorldMorph.

Instance Variables
	realDisplay:		the realdisplay
	wrapMorph:	the wrapped world
!

!AthensWrappedWorldDisplay class methodsFor: 'instance creation' stamp: 'NicolaiHess 1/4/2015 18:30'!
on: aWorldWrapMorph
	^ self new wrapMorph:aWorldWrapMorph! !

!AthensWrappedWorldDisplay methodsFor: 'display protocol' stamp: 'NicolaiHess 1/4/2015 18:30'!
boundingBox
	^ wrapMorph ifNotNil: [ wrapMorph fullBounds ] ifNil: [ 0 @ 0 extent: 0 @ 0 ]! !

!AthensWrappedWorldDisplay methodsFor: 'display protocol' stamp: 'NicolaiHess 1/4/2015 18:30'!
doesNotUnderstand: aMessage
	"delegate all not implemented methods to the real display"

	^ (realDisplay respondsTo: aMessage selector)
		ifTrue: [ realDisplay perform: aMessage selector withEnoughArguments: aMessage arguments ]
		ifFalse: [ super doesNotUnderstand: aMessage ]! !

!AthensWrappedWorldDisplay methodsFor: 'display protocol' stamp: 'NicolaiHess 1/4/2015 18:30'!
extent
	^ wrapMorph ifNotNil: [ wrapMorph extent ] ifNil: [ 0 @ 0 ]! !

!AthensWrappedWorldDisplay methodsFor: 'initialization' stamp: 'NicolaiHess 1/4/2015 18:30'!
initialize

	super initialize.

	realDisplay := Display.
	wrapMorph := nil.! !

!AthensWrappedWorldDisplay methodsFor: 'accessing' stamp: 'NicolaiHess 1/4/2015 18:30'!
wrapMorph
	^ wrapMorph! !

!AthensWrappedWorldDisplay methodsFor: 'accessing' stamp: 'NicolaiHess 1/4/2015 18:30'!
wrapMorph: anObject
	wrapMorph := anObject! !

!AthensBalloonSurface methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 6/24/2014 21:06'!
displayOnMorphicCanvas: canvas at: aPoint
	"display receiver on morphic canvas"
	canvas
		image: self asForm
		at: aPoint
		sourceRect: (0@0 extent: self asForm extent )
		rule: 34.! !

PluggableTextMorph subclass: #AthensPluggableTextMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Morphic'!

!AthensPluggableTextMorph commentStamp: '<historical>' prior: 0!
this class is subject of changes/removal. do not use it.!

!AthensPluggableTextMorph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/21/2011 12:01'!
textMorphClass
	^ AthensTextMorph! !

!BracketMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/15/2015 14:08'!
createClosedPolygonPathFrom: aPointsArray on: anAthensCanvas
	^ anAthensCanvas
		createPath: [ :builder | 
			aPointsArray ifEmpty: [ builder ].
			builder absolute.
			builder moveTo: aPointsArray first.
			aPointsArray allButFirstDo: [ :p | builder lineTo: p ].
			builder close.
			builder ]! !

!BracketMorph methodsFor: '*Athens-Morphic' stamp: 'NicolaiHess 1/15/2015 14:11'!
drawOnAthensCanvas: anAthensCanvas
	"Draw triangles at the edges."

	| r |
	r := self horizontal
		ifTrue: [ self bounds insetBy: (2 @ 1 corner: 2 @ 1) ]
		ifFalse: [ self bounds insetBy: (1 @ 2 corner: 1 @ 2) ].
	anAthensCanvas setPaint: self borderColor.
	anAthensCanvas drawShape: (self createClosedPolygonPathFrom: 
	(self leftOrTopVertices: self bounds) on: anAthensCanvas).
	
	anAthensCanvas setPaint: self fillStyle.
	anAthensCanvas drawShape: (self createClosedPolygonPathFrom: 
	(self leftOrTopVertices: r) on: anAthensCanvas).
	
	anAthensCanvas setPaint: self borderColor.
	anAthensCanvas drawShape: (self createClosedPolygonPathFrom: 
	(self rightOrBottomVertices: self bounds) on: anAthensCanvas).
	
	anAthensCanvas setPaint: self fillStyle.
	anAthensCanvas drawShape: (self createClosedPolygonPathFrom:
	(self rightOrBottomVertices: r) on: anAthensCanvas)! !
